from py4j.java_gateway import get_field, JavaGateway
from abc import abstractmethod

class FightingController:

    # ---- Do NOT override these methods ----

    class Java:
        implements = ["gameInterface.AIInterface"]

    def __init__(self, gateway):
        self.gateway = gateway # type: JavaGateway

        # Good practice to declare instance variables in __init__
        self.frameData = None # Updated after every call to getInformation
        self.inputKey = None
        self.cc = None
        self.player = None
        self.gameData = None
        self.simulator = None
        self.isGameJustStarted = True

    def getInformation(self, frameData):
        """
        Receives and stores frame data for use by other methods.
        :param frameData: data of the current frame.
        """
        self.frameData = frameData

    def initialize(self, gameData, player):
        """
        Prepares this AI for operation.
        :param gameData: information about the game the AI is about to enter
        :param player: boolean flag indicating whether this AI controls P1 or P2
        :return: 0 if the initialization is successful, the error code otherwise
        """
        self.inputKey = self.gateway.jvm.structs.Key()
        self.frameData = self.gateway.jvm.structs.FrameData()
        self.cc = self.gateway.jvm.commandcenter.CommandCenter()
            
        self.player = player
        self.gameData = gameData
        self.simulator = self.gameData.getSimulator()

        self.init()

        return 0

    def input(self):
        """
        Returns the command decided by the AI for the current frame.
        :return: the command decided by the AI for the current frame.
        """
        return self.inputKey


    def processing(self):
        """
        Delivers the input for the current frame. If no command is set, this
        :return:
        """
        if not self.frameData.getEmptyFlag() and self.frameData.getRemainingTime() > 0:

            # Let CommandCenter check if this is a new round - if so, the current skill should be canceled.
            self.cc.setFrameData(self.frameData, self.player)

            if not self.cc.getskillFlag():
                self.inputKey.empty()
                self.cc.skillCancel()
                command = self.computeCommand()
                if command is not None:
                    self.cc.commandCall(command)

            self.inputKey = self.cc.getSkillKey()


    # ---- DO override these methods ----

    def init(self):
        """
        Prepares this AI for operation.
        """
        pass


    @abstractmethod
    def getCharacter(self):
        """
        Returns the character to be controlled by this AI.
        Legal values are 'ZEN', 'GARNET', 'LUD', and 'KFM'.
        :return: the character to be controlled by this AI.
        """
        pass


    @abstractmethod
    def computeCommand(self):
        """
        Computes the next command to execute. This method is only called after the previous command is done executing.
        :return: the next command to execute.
        """
        pass


    def close(self):
        """
        Executes operations necessary before the AI is shut down (e.g., saving data etc.).
        """
        pass


class AI_NAME(FightingController):

    def getCharacter (self):
        return 'ZEN'

    def computeCommand (self):

        distance = self.cc.getDistanceX()

        my = self.cc.getMyCharacter()
        energy = my.getEnergy()
        my_x = get_field(my, 'x')
        my_state = get_field(my, "state")

        opp = self.cc.getEnemyCharacter()
        opp_x = get_field(opp, 'x')
        opp_state = get_field(opp, "state")

        xDifference = my_x - opp_x

        if (get_field(opp, "energy") >= 300) and ((get_field(my, "hp") - get_field(opp, "hp")) <= 300):
            return "FOR_JUMP _B B B"
        elif not my_state.equals(self.gateway.jvm.enumerate.State.AIR) and not my_state.equals(self.gateway.jvm.enumerate.State.DOWN):
            if distance > 150:
                return "FOR_JUMP"
            elif energy >= 300:
                return "STAND_D_DF_FC"
            elif (distance > 100) and (energy >= 50):
                return "STAND_D_DB_BB"
            elif opp_state.equals(self.gateway.jvm.enumerate.State.AIR):
                return "STAND_F_D_DFA"
            elif distance > 100:
                return "6 6 6"
            else:
                return "B"
        elif ((distance <= 150) and (my_state.equals(self.gateway.jvm.enumerate.State.AIR) or my_state.equals(self.gateway.jvm.enumerate.State.DOWN))
            and (((self.gameData.getStageXMax() - my_x) >= 200) or (xDifference > 0))
            and ((my_x >= 200) or xDifference < 0)):
            if energy >= 5:
                return "AIR_DB"
            else:
                return "B"
        else:
            return "B"
