import sys
import argparse
import pkgutil
from importlib import import_module
from py4j.java_gateway import JavaGateway, GatewayParameters, CallbackServerParameters

class GamesRunner:
    def __init__(self):
        self.args = self.parse_args()

        # Initialize communication channels with Java
        self.gateway = JavaGateway(
            gateway_parameters=GatewayParameters(port=self.args.port),
            callback_server_parameters=CallbackServerParameters(port=0)
        )
        python_port = self.gateway.get_callback_server().get_listening_port()
        self.gateway.java_gateway_server.resetCallbackClient(self.gateway.java_gateway_server.getCallbackClient().getAddress(), python_port)
        self.manager = self.gateway.entry_point

        # Run the games
        self.run_all_games()
        self.close_gateway()


    @staticmethod
    def parse_args():
        parser = argparse.ArgumentParser(
            description='Run a multi-episode experiment')
        parser.add_argument(
            '-n'
            '--number-of-games',
            dest='number_of_games',
            type=int,
            default=1,
            help='Number of games to play.')
        parser.add_argument(
            '-p1'
            '--player-1',
            dest='player_1',
            type=str,
            required=True,
            help="Name of the AI that will control player 1. This can be either the name of a jar package in the"
                 "engine's data/ai/ folder or the name of a class in one of the modules in fighting_ice/agents/ .")
        parser.add_argument(
            '-p2'
            '--player-2',
            dest='player_2',
            type=str,
            required=True,
            help="Name of the AI that will control player 2. This can be either the name of a jar package in the"
                 "engine's data/ai/ folder or the name of a class in one of the modules in fighting_ice/agents/ .")
        parser.add_argument(
            '-c1'
            '--character-1',
            dest='character_1',
            type=str,
            choices=['ZEN', 'GARNET', 'LUD', 'KFM'],
            help="Name of the character played by player 1. This can be 'ZEN', 'GARNET', 'LUD' or 'KFM'.")
        parser.add_argument(
            '-c2'
            '--character-2',
            dest='character_2',
            type=str,
            choices=['ZEN', 'GARNET', 'LUD', 'KFM'],
            help="Name of the character played by player 2. This can be 'ZEN', 'GARNET', 'LUD' or 'KFM'.")
        parser.add_argument(
            '-p'
            '--port',
            dest='port',
            type=int,
            default=4242,
            help='Number of games to play.')
        return parser.parse_args()


    def load_agent(self, agent_name):
        module_names = [name for _, name, _ in pkgutil.iter_modules(['fighting_ice/agents'])]
        for module_name in module_names:
            if not module_name.endswith('_agents'):
                continue
            module = import_module("fighting_ice.agents.%s" % module_name)
            if hasattr(module, agent_name):
                controller_class = getattr(module, agent_name)
                return controller_class(self.gateway)
        return None


    def run_one_game(self):
        controller_1 = self.load_agent(self.args.player_1)
        character_1 = controller_1.getCharacter()
        if controller_1 is not None:
            controller_1_name = self.args.player_1 + "1"
            self.manager.registerAI(controller_1_name, controller_1)
        else:
            controller_1_name = self.args.player_1

        controller_2 = self.load_agent(self.args.player_1)
        character_2 = controller_2.getCharacter()
        if controller_2 is not None:
            controller_2_name = self.args.player_2 + "2"
            self.manager.registerAI(controller_2_name, controller_2)
        else:
            controller_2_name = self.args.player_2

        game = self.manager.createGame(character_1, character_2, controller_1_name, controller_2_name)
        self.manager.runGame(game)

        sys.stdout.flush()

    def run_all_games(self):
        for i in range(self.args.number_of_games):
            print("Starting game %d" % (i + 1))
            self.run_one_game()
            print("Game %d finished" % (i + 1))

    def close_gateway(self):
        self.gateway.close_callback_server()
        self.gateway.close()

if __name__ == '__main__':
    GamesRunner()